<?php
/************************************************************
* Plugin Name:			Events Manager Pro - Gateway Selector
* Description:			Easily set or unset your activated payment gateway(s) per individual single event with a simple checkbox.
* Version:				1.3.7
* Author:  				Stonehenge Creations
* Author URI: 			https://www.stonehengecreations.nl/
* Plugin URI: 			https://www.stonehengecreations.nl/creations/stonehenge-em-gateway-selector/
* License URI: 			https://www.gnu.org/licenses/gpl-2.0.html
* Text Domain: 			stonehenge-em-gateway-selector
* Domain Path: 			/languages
* Requires at least: 	5.2
* Tested up to: 		5.4
* Requires PHP:			7.2
* Network:				false
************************************************************/

// Exit if accessed directly
if( !defined('ABSPATH') ) exit;
include_once(ABSPATH.'wp-admin/includes/plugin.php');


Class Stonehenge_EM_Gateway_Selector {

	#===============================================
	public function __construct() {
		$this->slug = 'stonehenge-em-gateway-selector';
		add_action('init', array($this, 'load_translations'));
		add_action('em_events_admin_bookings_footer',array($this, 'add_selector_to_tickets'), 50, 1);
		add_filter('em_event_validate_meta', array($this,'validate_gateway_selection'),10,2);
		add_action('save_post', array($this, 'save_em_active_gateways'));
		add_action('em_booking_form_footer', array($this, 'process_em_active_gateways'), 9);
		add_filter('plugin_row_meta', array($this, 'create_additional_plugin_links'), 10, 2);
	}


	#===============================================
	# 	Show Gateway Selector in Front-End Event Submission Form.
	#===============================================
	function add_selector_to_tickets($EM_Event) {
		echo '<br style="clear: both;">';
		echo '<h4>' . __('Payment Options', 'stonehenge-em-gateway-selector') . '</h4>';
		echo '<div id="em-gateway-selector">';
		echo '<p><em>'. __('Please select which payment method(s) you would like to be available for this event.','stonehenge-em-gateway-selector').'</em></p>';

		global $EM_Gateways; 								// Get access to all registered Gateways.
		$active_gateways = EM_Gateways::active_gateways();	// Get activated gateways.
		if( empty($active_gateways) ) {
			echo '<span style="color:#e14d43;">' . __('No Payment Gateways activated.') .'</span>';
		}
		else {
			asort($active_gateways);						// Sort gateways alphabetically.
			$post_id = $EM_Event->post_id;

			wp_nonce_field(basename(__FILE__), '_em_active_gateways_nonce');

			// New event, nothing saved yet.
			if(!metadata_exists('post', $post_id, '_em_active_gateways')) {
				$saved_selection = array();
			}
			else {
				// Get saved values.
				$saved_selection = maybe_unserialize( get_post_meta($post_id, '_em_active_gateways', true));
			}

			foreach($active_gateways as $id => $name) {
				$checked = (!isset($EM_Event->event_id) || in_array($id, $saved_selection)) ? 'checked' : '';
				echo "<p><label for='{$id}'><input type='checkbox' value='{$id}' id='{$id}' name='gateways[]' {$checked}>{$name}</label></p>";
			}
		}
		echo '</div>';
	}


	#===============================================
	# 	Prevent disabling all gateways (Form validation).
	#===============================================
	public function validate_gateway_selection($result, $EM_Event) {
		$EM_Tickets = $EM_Event->get_tickets();

		// Are there tickets?
		if( count($EM_Tickets) > 0 ) {
			foreach( $EM_Tickets as $EM_Ticket ) {
				// Only proceed of tickets are not free.
				if( $EM_Ticket->ticket_price != '0.00') {
					// There are paid tickets, so we need to select a gateway.
					if( !isset($_POST['gateways']) || !is_array($_POST['gateways']) ) {
						$EM_Event->add_error( __('You need to select at least one Payment Gateway.', 'stonehenge-em-gateway-selector'));
						return false;
					}
				}
			}
		}
		return $result;
	}


	#===============================================
	# 	Save Gateway Selection to WP Database.
	#===============================================
	public function save_em_active_gateways($post_id) {
		$is_autosave 	= wp_is_post_autosave($post_id);
		$is_revision 	= wp_is_post_revision($post_id);
		$is_valid_nonce = (isset($_POST['_em_active_gateways_nonce']) && wp_verify_nonce($_POST['_em_active_gateways_nonce'], basename(__FILE__))) ? 'true' : 'false';

		if($is_autosave || $is_revision || !$is_valid_nonce) {
			return;
		}

		// Only process if bookings are enabled.
		if( isset($_POST['event_rsvp']) && isset($_POST['gateways']) && !empty($_POST['gateways'])) {
			$checkboxes = array_map('sanitize_text_field', wp_unslash($_POST['gateways']));
			update_post_meta($post_id, '_em_active_gateways', $checkboxes);
		}
	}


	#===============================================
	# 	Hook into the Booking Form Footer to only show selected Gateways.
	#===============================================
	public function process_em_active_gateways() {
		global $EM_Gateways;
		$active_gateways = EM_Gateways::active_gateways();
		$saved_selection = maybe_unserialize( get_post_meta(get_the_ID(), '_em_active_gateways', true));

		if( empty($active_gateways) ) {
			return;
		}
		elseif( isset($saved_selection) && !empty($saved_selection) && is_array($saved_selection) ) {
			foreach( $active_gateways as $id => $name ) {
				if( !in_array($id, $saved_selection) ) {
					unset($EM_Gateways[$id]);
				}
			}
		}
	}


	#===============================================
	# 	Make this Plugin translatable.
	#===============================================
	public function load_translations() {
		$text 	= 'stonehenge-em-gateway-selector';
		$file 	= WP_PLUGIN_DIR .'/'. $text;
		$locale = apply_filters( 'plugin_locale', function_exists( 'determine_locale' ) ? determine_locale() : get_locale(), $text );
		$mofile = ( $file ) . '/languages/'. $text . '-' . $locale . '.mo';
		$loaded = load_textdomain( $text, $mofile );
		if( !$loaded ) { $loaded = load_plugin_textdomain( $text, false, '/languages/' ); }
		if( !$loaded ) { $loaded = load_muplugin_textdomain( $text, '/languages/' ); }
	}


	#===============================================
	# 	Add additional links to WP Plugins Page.
	#===============================================
	public function create_additional_plugin_links($links, $file) {
		$plugin = plugin_basename(__FILE__);
		if ($file == $plugin) { // Only for this plugin
			$plugindir  = explode("/plugins/", dirname(__FILE__));
			$wpslug		  = $plugindir[1];
			$author		  = 'DuisterDenHaag';
			$donate		  = 'https://useplink.com/payment/VRR7Ty32FJ5mSJe8nFSx';
			return array_merge(
				$links,
				array('<a href="https://wordpress.org/support/plugin/'.$wpslug.'" target="_blank">' . __('Plug-in Support', $this->slug). '</a>'),
				array('<a href="https://profiles.wordpress.org/'.$author.'" target="_blank">' . __('WordPress Profile', $this->slug) . '</a>'),
				array('<a href="'.$donate.'" target="_blank">' . __('Donate', $this->slug) . '</a>')
			);
		}
		return $links;
	}

} // End class.

#===============================================
add_action('plugins_loaded', function() {
	if( !is_plugin_active('events-manager-pro/events-manager-pro.php') ) {
		deactivate_plugins(plugin_basename(__FILE__));
		wp_die('This add-on needs Events Manager Pro to be installed and activated.', 'Plugin dependency check', array('back_link' => true) );
	}
	else {
		new Stonehenge_EM_Gateway_Selector();
	}
}, 10);
